'========================================
                                        |
'  x3d functions                        |
                                        |
'---------------------------------------|
' update 05:17 24/08/2007               |
                                        |
'========================================


'========================================
                                        |
function createPolyframe _              | main function with default parameters
( _          |                          |
 na      ,_  | name                     |
 n=6     ,_  | polygon factor           | points to define polygon
 ra=2    ,_  | radius                   | radius in metres
 wi=0.1  ,_  | member width             | width of frame members
 th=.022 ,_  | member thickness         | thickness of frame or cladding
 pm=0    ,_  | alternation mode         | 
 lap=0   ,_  | face overlap flag        |
 ti1=6   ,_  | number of frames at base |
 tq=4    ,_  | number of tiers          |
 sl=90   ,_  | start slope angle        |
 si=12   ,_  | slope decrement          |
 hp=.5   ,_  | panel length from tail   |
 ovl=0   ,_  | overlap value            |
 ma=""   ,_  | materialto be used       |
 im=""    _  | image for material texture
)                                       |
  numbers("5")                          |
                                        |
'---------------------------------------'
| VARIABLES                             |
'---------------------------------------'
  new s,t,u,v                           | strings for x3d coord lists
  new r,a,b,ssd,ai=360/n, x,y,z1,z2     | ai base angle step 
  new tx,ty,txi,pf=pm and 1             | tx ty coord index strings
  new l=0,lt=0, d=0,i=0,c=0             |
  new inr=0,mx=n*2,a1,a2,ad             |
  new aj=0,oo=0                         |
  new sr,sj                             |
  new ri,ba=0,ta=0,hs,ss,vs,ws,hd,ga    | tail length
  new hdtrim=0,tc,fuse=0                |
  new altp=0,altpa=0,hx=0               | hx hexagon orientation
  new poa=180/ti1                       | base frame chord angle
  new por=tand(poa),por2=tand(poa*2)    | base frame half chord and full chord
  new tt=""                             |
  new gr=0                              |
'---------------------------------------|
  'new ps[12]                           | for x y polygon points
  new ps=12                             | these vars must be contiguous
  new px1,py1,px2,py2,px3,py3,px4,py4   |
  new px5,py5,px6,py6                   |
'---------------------------------------|
  set la+=1                             | layer counter
  out (o3,cr+"Layer"+str la+cr)         | layers of structure innner skin/frame/outer etc
  out (o2,"")                           | set to x3d output
  set tq-=(redu*2)                      | decrement tier count for turret and or cap
'---------------------------------------'



|---------------------------------------|
| POLYGON ARRAY                         | default base values
|---------------------------------------|
                                        | Parameter block for building the frames
  ps =  mx                              | number of points*2
  px1 = -ra*por; py1=0                  | lower right
  px2 = .0; py2=-.2                     |
  px3 = -px1; py3=0                     | lower left
  px4 = .3; py4=hp                      |
  px5 = -.0; py5=hp+0.2                 |
  px6= -.3; py6=hp                      |
                                        |
'---------------------------------------|


|---------------------------------------|
| MAIN LOOP                             |
|---------------------------------------|
  b=.5                                  |
  $ <Transform translation="0 -1 0">    | 
  tc=0                                  | set tier counter to 0
  {                                     | loop for each tier of frames
   tc++; if tc GT tq then exit          | tier counter
   //if tc EQ tq then hdtrim=1          | set trim flag if last tier
   if redu then if tc EQ tq then fuse=1 | fuse on last tier
   do tier                              | generate frames for this tier
   repeat                               | reapeat for all tiers
  }                                     | frame completed
  $   </Transform>                      | end of function
 end                                    |
'---------------------------------------|


'---------------------------------------|
 def tier                               | for each tier
                                        |
  if pm LT 2 then pf=pm                 | alternation
  if pm GT 1 then pf=(1-pf)             |
                                        | calc poly frame
  if (ps EQ 12)                         | are thers 12 member points defineing the frame?
                                        |
'---------------------------------------| assume symmetric 
   py4=py3+hp-ta                        | base to shoulder length
   py6=py4                              | other shoulder
   ga=atand(cosd(sl)*por,1)             | taper angle
   px4=px3-(hp-ta*tand(ga))             | right shoulder width
   px6=px1+(py6-py1 *tand(ga))          | left shoulder width
   ri=atand(por*sind(sl),1)             | ridge angle
   py2=py3-ta                           | base to tail (negative)
'---------------------------------------|
   a=sind(ga) / sind(poa)               |
   sr=atand(pyth(1,a,-1),a)             | acosd // ridge slope
   sj=sr-sl+si                          | slope on next tier
   ba=ta;ta=px4*sind(ri)/sind(sj)       |
   hs=ta*cosd(sj)                       |
   vs=px4/cosd(ga)-(hs*tand(ga))        |
   ss=pyth(vs,hs,1)                     | edge
   ws=pyth(ss,ta,-1)                    |
   hd=pyth(ss,px4,-1)                   |
'---------------------------------------|
                                        | parameters for the fusing hexagon
   if fuse then                         |
    a=(1/tand(sl-si))_                  |
    -(cosd(poa*2)/tand(sl))             |
    hs=px4*sind(poa*2)/a                |
    hd=hs/sind(sl);ss=pyth(hd,px4,1)    |
   end if                               |
'---------------------------------------|
   if hdtrim then hd=0;hdtrim=0         | cancel trim mode
   py5=py4+hd                           |
  end if                                |
'---------------------------------------|
  set hts=py5*sind(sl)                  |
  tt="A";do dataReport1                 | report dimensions apex ended hexagon
'---------------------------------------|
  if (tc EQ 1)or(tc EQ 3) then          |
   altp=1 and not fuse                  | set akternation flag
  end if                                |
  b=-1/por*px3                          |
'---------------------------------------|
  do tierFaces                          | construct the main tier faces
'---------------------------------------|
                                        |
  if altp then                          | alternating faces
   altpa=1;oo+=2;b=-1/por*px3           | set alternation flag and angle offset
'---------------------------------------|
   do tierFaces;oo-2;altpa=0            | do hexagon faces
'---------------------------------------|
  end if                                |
  altp=0                                |
                                        |
  if (fuse EQ 1)and(ps EQ 12) then      |
   oo+=2                                | alternation angle offset
   py5=py4                              | flatten apex to give pentagon
'---------------------------------------|
   set hts=py5*sind(sl)                 |
   tt="B";do dataReport1                | report face type 1 parameters
   b=-1/por*px3                         |
'---------------------------------------|
   do tierFaces                         | do Pent
'---------------------------------------| 
                                        |
   fuse=2                               |
  end if                                |
  if ps EQ 12                           | ps is normally 12
   set hti+=(py5*sind(sl))              | height before next stage
'---------------------------------------|
   if fuse EQ 2 then                    | halved number of faces per tier
    hx=1                                | hexagon oriented with flat base and top
    ta=hs/sind(sl-si)                   | tail length ??
    ws=pyth(ss,ta,-1)                   | connecting edge ??
    ga=atand(por2*cosd(sl-si),1)        |
    px1=-px4;px2=px4                    |
    py1=0;py2=py1                       |
    px3=px2+ws;px6=px1-ws               |
    py3=py1+ta;py6=py3                  |
    px4=px3-(hp*.50*tand(ga))           |
    set pra=px4/tand(poa*2)             |
    py4=py3+(hp*.50)                    |
    px5=-px4;py5=py4                    |
    set fuse=1                          |
    set sl-=si                          |
'---------------------------------------|
    set hts=py5*sind(sl)                |
    tc+=1;do dataReport2                | report the dimensions of fuser hexagon
'---------------------------------------|
    altpa=1                             |
    b=-1/por*px2                        |
'---------------------------------------|
    do tierFaces                        |
'---------------------------------------|
    ws=px4                              |
    set ovl=0                           |
    hx=0                                |
'---------------------------------------|
    if turret                           | is a turret required?
     set hti+=(py5*sind(sl))            | height so far
'---------------------------------------|
     set sl=80                          |
     px1=-ws;py1=0                      |
     px2=0;py2=0;px3=ws;py3=0           |
     ga=atand(por2*cosd(sl),1)          |
     ws=ws-(hp*tand(ga));py4=py3+hp     |
     px4=ws;px6=-ws                     |
     px5=0;py5=py4;py6=py4              |
'---------------------------------------|
     set hts=py5*sind(sl)               |
     tc+=1;do dataReport3               | report parameters for face type 3 (turret)
'---------------------------------------|
     altpa=1                            |
'---------------------------------------|
     b=px1/por2;do tierFaces            |
'---------------------------------------|
    end if                              |
    if cap                              | is a cap required?
     set hti+=(py5*sind(sl))            | height so far
'---------------------------------------|
     set sl=15                          |
     px1=-ws;px3=ws;px2=0               |
     r=ws/por2                          |
     set hp=r/cosd(sl)                  |
     px4=0;px5=0                        |
     px6=0;py1=0;py2=0;py3=0            |
     py4=hp;py5=hp;py6=hp               |
'---------------------------------------|
     set hts=py5*sind(sl)               |
     tc+=1;do dataReport4               |
'---------------------------------------|
     altpa=0                            |
     pf=0                               |
'---------------------------------------|
     b=px1/por2;do tierFaces            |
'---------------------------------------|
    end if                              |
    fuse=0                              | cancel the fusion flage
   end if                               | end fusion
'---------------------------------------| 
   set sl-=si                           | reduce slope for next tier
   oo=1-oo                              | alternate the rotation displacement
   px3=ws; px1=-ws                      | set right and left hip widths
  end if                                | end =12
 end                                    | end tier subroutine
'---------------------------------------|

'---------------------------------------|
 def dataReport1                        | Hexagon with Aoex and tail, top and bottom
  iquotes < >                           | because square brackets are used in function
   out(o3,cr)                           | select report file
   if fuse EQ 0 then $ Tier <str tc>    | tier level
   if fuse EQ 1 then $ Tier <str tc+tt> | alternating faces ??
   if fuse EQ 0 then $  faces <str ti1> | nuber of faces in tier 
   if fuse EQ 1 then $  faces <str(ti1/2)> half number of faces
   $  hb  length base   <py2*sc>        | tail value
   $  hl  length lower  <py3*sc>        | length from tail to hip
   $  hm  length middle <py4*sc>        | length from hip to shoulder
   $  hu  length upper  <py5*sc>        | length from hip to apex
   $  wb  width base    <px3*sc>        | width from centre to hip
   $  wu  width upper   <px4*sc>        | width from centre to shoulder
   $  hti height increm <hti+hts*sc>    | contribution to height of dome
   out(o2,"")                           | revert to x3d script output
  iquotes [ ]                           | revert to normal iquotes
 end                                    |
'---------------------------------------|

'---------------------------------------|
 def dataReport2                        | hexagon flat side top and bottom
  iquotes < >                           | use alternative iquotes so we can involve square brackets
   out(o3,cr)                           | output to parameter file
   $ Tier <str tc>                      | tier number
   $  faces <str(ti1/2)>                | number of faces = half base count
   $  hl  length lower  <py3*sc>        | length from base to middle
   $  hu  length upper  <py4*sc>        | length from middle to top
   $  wb  width base    <px2*sc>        | width from centre to base
   $  wm  width middle  <px3*sc>        | width from centre to middle apex
   $  wu  width upper   <px4*sc>        | width from centre to top shoulder
   $  hti height increm <hti+hts*sc>    | height contribution to dome
   out(o2,"")                           | revert to x3d script
  iquotes [ ]                           | revert to normal iiquotes
 end                                    |
'---------------------------------------|

'---------------------------------------|
 def dataReport3                        | for turret measurements
  iquotes < >                           | change iquotes
   out(o3,cr)                           | switch $ output to data file
   $ Tier <str tc> (turret)             |
   $  faces <str(ti1/2)>                | faces= base faces/2
   $  hu  length upper  <py5*sc>        | length of turret sides
   $  wb  width base    <px3*sc]>       | width from centre to base
   $  wu  width upper   <px4*sc>        | width from centre to top (shoulder)
   $  hti height increm <hti+hts*sc>    | height contribution to dome
   out(o2,"")                           | revert to x3d output
  iquotes [ ]                           | revert to normal iquotes
 end                                    |
'---------------------------------------|


'---------------------------------------|
 def dataReport4                        | for cap measurements
  iquotes < >                           | change iquotes
   out(o3,cr)                           | switch $ output to data file
   $ Cap                                |
   $ Tier <str tc>                      |  tier number
   $  faces <str(ti1/2)> (cap)          |  number of faces
   $  hu  length upper  <py5*sc>        |  length of cap
   $  wb  width base    <px3*sc>        | width from centre to base
   $  hti height increm <hti+hts*sc>    | height contribution to dome
   out(o2,"")                           | revert to x3d output
  iquotes [ ]                           | revert to normal iquotes
 end                                    |
'---------------------------------------|


'-----------------------------------------------|
 def tierFaces                                  |
                                                |
  set trc+=1                                    |
                                                |
  $ <Transform translation="0 [hti] 0">         |
  $ <Group DEF="[na]">                          |
  $ <Transform rotation="0 1 0 [oo*pi/ti1+ska]">|
  $ <Transform DEF='TRANS[trim(str trc)]'>      |
  $ <Transform translation="0 0 [b]">           |
  $ <Transform rotation="1 0 0 [90-sl*pi/180]"> |
  $ <Shape>                                     |
                                                |
       Appearance(ma,im)                        | specify colours and texture image
                                                |
  $ <IndexedFaceSet DEF="polyindex" solid="true"|
  if pf EQ 0                                    |
   ssd=PolyPanelCoordIndex(n,pf)                |
   $     [ssd]                                  |
  end if                                        |
  if pf EQ 1 then segCoordIndex(n,0)            |
  $                                             |
  do insertcoordinates                          | coordinate set and index set per polygon frame or panel
  $                                             |
  $ </IndexedFaceSet>                           |
  $ </Shape>                                    |
//$ </Transform>                                |
  $                                             |
  if pf EQ 1                                    |
   i=0                                          |
   {                                            |
    i++; if i GE n then exit                    |
    do nextseg                                  | next frame or panel
    repeat                                      |
   }                                            |
  end if                                        |
  $ </Transform>                                |
  $ </Transform>                                |
  $ </Transform>                                |
  $ </Transform>                                |
  $ </Group>                                    |
  aj=0                                          |
  numbers("3")                                  |
  {                                             |
   aj+=((fuse+1)*(altp+1)*poa*2)                |
   if aj GE 360 then exit                       |
   $ <Transform rotation="0 1 0 [aj*pi/180]">   |
   $ <Group USE="[na]"/>                        |
   $ </Transform>                               |
   repeat                                       |
  }                                             |
  $ </Transform>                                |
 end                                            |
'-----------------------------------------------|


'-----------------------------------------------|
 def nextseg                                    | for each frame or panel
  $ <Shape>                                     |
  $ <Appearance USE="A[ma]"/>                   |
  $ <IndexedFaceSet solid="true"                |
    segCoordIndex(n,i)                          |
  $ <Coordinate USE="polycoords"/>              |
  if im then                                    |
   $ <TextureCoordinate USE="polytexcoords"/>   |
  end if                                        |
  $ </IndexedFaceSet>                           |
  $ </Shape>                                    |
  $                                             |
 end                                            |
'-----------------------------------------------|
                                                |
'-----------------------------------------------|
 def insertcoordinates                          | construct coordinate and index lists 
  $ <Coordinate DEF="polycoords" point="        |
                                                |
  r=.5; a=ai/2; s,t,u=" "; do makeCoordinates   |
  $ [s]                                         |
  $ [t]                                         |
  if pf                                         |
   inr=1;r=.5-(r*wi/cosd(ai/2));s,t,u=" "       |
   a=ai/2; do makeCoordinates;inr=0             |
   $ [s]                                        |
   $ [t]                                        |
  end if                                        |
                                                |
  $ "/>                                         |
  if im then                                    |
   $ <TextureCoordinate DEF="polytexcoords" point="
   $ [u]                                        |
   $ [v]                                        |
   $ "/>                                        |
  end if                                        |
 end                                            |
'-----------------------------------------------|

'-----------------------------------------------|
 def makeCoordinates                            | make face coordinates and texture mapping coordinates
  // with mirrored texture for the rear face    |
  c=0                                           |
  {                                             |
   numbers("3")                                 |
   if ps EQ 0 then x=r*sind(a);y=-r*cosd(a)     |
   if ps then                                   |
    x=ps[c+1];y=ps[c+2]                         |
    if inr                                      |
     //calc inner coords                        |
     c+=mx                                      |
     a1=atand(ps[c+1 mod mx+1]-ps[c-1 mod mx+1],_
     ps[c+0 mod mx+1]-ps[c-2 mod mx+1])         |
     a2=atand(ps[c+3 mod mx+1]-ps[c+1 mod mx+1],_
     ps[c+2 mod mx+1]-ps[c+0 mod mx+1])         |
     c-=mx                                      |
     if lap EQ 0 then                           |
      ad=a2-a1 mod 360 /2                       |
      r=wi/cosd(ad); x=cosd(a1+ad+90)*r+x       |
      y=sind(a1+ad+90)*r+y                      |
     end if                                     |
     if lap EQ 1 then                           |
      ad=a2-a1 mod 360; r=wi/cosd(ad)           |
      x=cosd(a2)*r+x; y=sind(a2)*r+y            |
     end if                                     |
    end if                                      |
    if (pf EQ 0)and(ovl) then                   |
     if hx EQ 0 then                            |
      if c EQ 0 then                            |
       x=px1-px6*ovl+px6                        |
       y=py1-py6*ovl+py6                        |
      end if                                    |
      if c EQ 2 then y+=(py1-py6*(ovl-1))       |
      if c EQ 4 then                            |
       x=px3-px4*ovl+px4                        |
       y=py3-py4*ovl+py4                        |
      end if                                    |
     end if                                     |
     if hx EQ 1 then                            |
      if c EQ 10 then                           |
       x=px6-px5*ovl+px5                        |
       y=py6-py5*ovl+py5                        |
      end if                                    |
      if c EQ 0 then y=py1-py5*ovl+py5          |
      if c EQ 2 then y=py2-py4*ovl+py4          |
      if c EQ 4 then                            |
       x=px3-px4*ovl+px4                        |
       y=py3-py4*ovl+py4                        |
      end if                                    |
     end if                                     |
    end if                                      |
    c+=2                                        |
   end if                                       |
   z1=0;z2=-th                                  |
   s+="[x] [y] [z1], "                          |
   t+="[x] [y] [z2], "                          |
   l++;  if l GE 4 then s+=cr;t+=cr;l=0         |
   if pf EQ 0 then                              |
    tx=(x+.5); ty=y+1; txi=1-tx                 |
    u+="[tx] [ty], "                            |
    v+="[txi] [ty], "                           |
    lt++; if lt GE 4 then u+=cr;v+=cr;lt=0      |
   end if                                       |
   a+=ai; if a GT 360 then exit // GT           |
   repeat                                       |
  }                                             |
  if pf EQ 1 then                               |
   u="0 0,1 0,1 1,0 1,";v="1 0,0 0,0 1,1 1,"    |
  end if                                        |
 end                                            |
                                                |
'-----------------------------------------------|
end function                                    |
'-----------------------------------------------|


'-----------------------------------------------|
  function segCoordIndex(n,i)                   | generate coordinate indexes
   new n1=n,n2=n*2,n3=n*3                       |
   new c1,c2,c3,c4                              |
   numbers("0")                                 |
   $ coordIndex="                               |
   c1=mod(i+0,n)+n2; c2=mod(i+0,n)              |
   c3=mod(i+1,n); c4=mod(i+1,n)+n2              |
   $ [c1] [c2] [c3] [c4] -1                     |
   c1=mod(i+0,n)+n1; c2=mod(i+0,n)+n3           |
   c3=mod(i+1,n)+n3; c4=mod(i+1,n)+n1           |
   $ [c1] [c2] [c3] [c4] -1                     |
   c1=mod(i+0,n); c2=mod(i+0,n)+n1              |
   c3=mod(i+1,n)+n1; c4=mod(i+1,n)              |
   $ [c1] [c2] [c3] [c4] -1                     |
   c1=mod(i+1,n); c2=mod(i+1,n)+n1              |
   c3=mod(i+1,n)+n3; c4=mod(i+1,n)+n2           |
   $ [c1] [c2] [c3] [c4] -1                     |
   c1=mod(i+0,n)+n3; c2=mod(i+0,n)+n2           |
   c3=mod(i+1,n)+n2; c4=mod(i+1,n)+n3           |
   $ [c1] [c2] [c3] [c4] -1                     |
   c1=mod(i+0,n)+n1; c2=mod(i+0,n)              |
   c3=mod(i+0,n)+n2; c4=mod(i+0,n)+n3           |
   $ [c1] [c2] [c3] [c4] -1"                    |
   if im then                                   |
    $ texCoordIndex="                           |
    $ 0 1 2 3 -1 0 1 2 3 -1 0 1 2 3 -1          |
    $ 0 1 2 3 -1 0 1 2 3 -1 0 1 2 3 -1"         |
   end if                                       |
   $ >                                          |
  end function                                  |
'-----------------------------------------------|


'-----------------------------------------------|
  function PolyPanelCoordIndex(n,m=0)           | generated indexes for panels
                                                |
   new s="",t="",i, p=n*2                       |
                                                |
                                                | define front and back faces first 
   if not m                                     |
    i=0                                         |
    {                                           |
     s+=" "+str(i)                              |
     if im then t+=" "+str(i)                   |
     i++; if  i GE n then exit                  |
     repeat                                     |
    }                                           |
    s+=" -1"                                    |
    if im then t+=" -1"                         |
    i=0                                         |
    {                                           |
     //s+=" "+str(i); t+=" "+str(i+n)           |
     s+=" "+str(mod(n-i-1,n)+n)                 |
     if im then t+=" "+str(i+n)                 |
     i++; if  i GE n then exit                  |
     repeat                                     |
    }                                           |
    s+=" -1"                                    |
    if im then t+=" -1"                         |
   end if                                       |
                                                |
   if m                                         |
    i=0                                         |
    {                                           |
     s+=" "+str(i+p)+" "+str(i)+" "+str(i+1) _  |
     + " "+str(i+p+1)+" -1 "                    |
     if im then                                 |
      t+=" "+str(n*2+0)+" "+str(n*2+1) _        |
      +" "+str(n*2+2)+" "+str(n*2+3)+" -1 "     |
     end if                                     |
     i++; if  i GE n then exit                  |
     repeat                                     |
    }                                           |
    i=0                                         |
    {                                           |
     s+=" "+str(i)+" "+str(i+p) _               |
     +" "+str(i+1+p)+" "+str(i+1)+" -1 "        |
     if im then                                 |
      t+=" "+str(n*2+0)+" "+str(n*2+1) _        |
      +" "+str(n*2+2)+" "+str(n*2+3)+" -1 "     |
     end if                                     |
     i++; if  i GE n then exit                  |
     repeat                                     |
    }                                           |
   end if                                       |
                                                |
                                                | define outward surfaces
   i=0                                          |
   {                                            |
    s+=" "+str(mod(i,n))+" "+str(mod(i,n)+n) _  |
    +" "+str(mod(i+1,n)+n)+" "+str(mod(i+1,n))_ |
    +" -1 "                                     |
    if im then                                  |
     t+=" "+str(n+0)+" "+str(n+1)+" " _         |
     +str(n+2)+" "+str(n+3)+" -1 "              |
    end if                                      |
    i++; if  i GE n then exit                   |
    repeat                                      |
   }                                            |
                                                |
                                                | define inward surfaces
   if m                                         |
    s+=cr; t+=cr                                |
    i=0                                         |
    {                                           |
     s+=" "+str(mod(i,n)+" "+p+n)+" " _         |
     +str(mod(i,n)+p)+" "+str(mod(i+1,n)+p) _   |
     +" "+str(mod(i+1,n)+p+n)+" -1 "            |
     if im then                                 |
      t+=" "+str(n+0)+" "+str(n+1)+" " _        |
      +str(n+2)+" "+str(n+3)+" -1 "             |
     end if                                     |
     i++; if  i GE n then exit                  |
     repeat                                     |
    }                                           |
   end if                                       |
                                                |
   s=alignData(s,3,80)                          |
   if im then                                   |
    t=cr+"texCoordIndex="+qu+cr _               |
    +alignData(t,3,80)+cr+qu                    |
   end if                                       |
   return "coordIndex=    "+qu+cr+s+qu+t+">"    |
                                                |
  end function                                  |
                                                |
'-----------------------------------------------|

'-----------------------------------------------|
function alignData(s,n,e)                       | configure line of data into several lines with length e and margin n
 new u="", w="", c=0                            |
 read(s)                                        |
 {                                              |
  w=word(); if not w then exit                  |
  if w EQ "-" then w+=word()                    |
  u+=many(" ",n-len w)+w; c+=n                  |
  if (c GT e)and(w EQ "-1") then c=0; u+=cr     |
  repeat                                        |
 }                                              |
 return u                                       |
end function                                    |
'-----------------------------------------------|


'-----------------------------------------------|
function Appearance(ma="",im="")                | colour and texture schemes
 $      <Appearance DEF="A[ma]">                |
 {                                              |
  if ma EQ "timber"                             |
   $ <Material ambientIntensity='0.2' _         |
   diffuseColor='0.35 0.35 0.0' _               |
   shininess='0.08' _                           |
   specularColor='0.2 0.2 0.1'/>                |
   if im then $ <ImageTexture url='"[im]"' />   |
   exit                                         |
  end if                                        |
  if ma EQ "glass"                              |
   if altpa EQ 0 then                           |
    $ <Material ambientIntensity="0.2" _        |
    diffuseColor="0.25 0.55 0.75" _             |
    emissiveColor="0.5 0 0"                     |
    $ shininess="0.6" _                         |
    specularColor="0.8 0.8 0.95" _              |
    transparency="0.0" />                       |
   end if                                       |
   if altpa EQ 1 then                           |
    $ <Material ambientIntensity="0.2" _        |
    diffuseColor="0.25 0.55 0.65" _             |
    emissiveColor="0.5 0 0"                     |
    $ shininess="0.6" _           |
    specularColor="0.8 0.8 0.8" _               |
    transparency="0.5" />                       |
   end if                                       |
 //$ <Material ambientIntensity='0.2' _         |
   diffuseColor='0.35 0.35 0.0' _               |
   shininess='0.08' _                           |
   specularColor='0.2 0.2 0.1'/>                |
   exit                                         |
  end if                                        |
  //default                                     |
  $ <Material ambientIntensity='0.2' _          |
  diffuseColor='0.25 0.55 0.35' _               |
  shininess='0.08' _                            |
  specularColor='0.1 0.2 0.2'/>                 |
 }                                              |
 $ </Appearance>                                |
end function                                    |
'-----------------------------------------------|

